options("h2o.use.data.table" = TRUE)
options(stringsAsFactors = FALSE)
options(java.parameters = "-Xmx4096m")
options(dplyr.width = Inf) 
Sys.setenv(TZ='GMT')

library(tidyverse)
library(rpart)
library(ranger)
library(h2o)
library(class)
library(tictoc)
library(gridExtra)
source("functions.R")
h2o_srv = h2o.init()


n = 1e4
noise = 0.1
grid = crossing(x = seq(from=-1, to=1, length.out = sqrt(n) ),y = seq(from=-1, to=1, length.out = sqrt(n) ))


# DATASETS
datasets = list()

datasets[["01 diagonal"]] = mutate(grid, 
                               class = .9*x + .2 > y,
                               train = ifelse(rbinom(n,1,noise),!class,class))

datasets[["02 chess"]] = mutate(grid, 
                               class = sign(x) == sign(y),
                               train = ifelse(rbinom(n,1,noise),!class,class))

datasets[["03 cruz"]] = mutate(grid, 
                            class = abs(x) > abs(y),
                            train = ifelse(rbinom(n,1,noise),!class,class))

datasets[["04 chess2"]] = mutate(grid, 
                               class = sign(sin(6*x)) == sign(cos(6*y)),
                               train = ifelse(rbinom(n,1,noise),!class,class))

datasets[["05 harley queen"]] = mutate(grid, 
                                 class = abs(sin(6*x)) > abs(cos(6*y)),
                                 train = ifelse(rbinom(n,1,noise),!class,class))

datasets[["06 circulo"]] = mutate(grid, 
                                       class = sqrt(x^2+y^2) < 0.7,
                                       train = ifelse(rbinom(n,1,noise),!class,class))

datasets[["07 circulos"]] = mutate(grid, 
                                  class = sqrt(abs(sin(12*x)+sin(12*y))) > 0.8,
                                  train = ifelse(rbinom(n,1,noise),!class,class))

for(set in names(datasets)){
  tic("main")
  print(set)
  
  dataset = datasets[[set]]
  
  gr_train = ggplot(data = dataset, aes(x=x, y=y, fill=train)) + geom_tile() + ggtitle(paste(set,"train"))
  ggsave(paste0(set,"_train.png"),gr_train)
  
  gr_test = ggplot(data = dataset, aes(x=x, y=y, fill=class)) + geom_tile()+ ggtitle(paste(set,"test"))
  ggsave(paste0(set,"_test.png"),gr_test)

  #MODELS
  models = list()
  
  tic("logit")
  fit = glm(train ~  x + y, dataset,family = binomial())
  toc()
  models[["logit"]] = mutate(dataset,
                             score = fit$fitted.values,
                             prediction = round(score) == 1)
  
  tic("tree")
  fit = rpart(train ~  x + y, dataset)
  toc()
  models[["tree"]] = mutate(dataset,
                            score = predict(fit,dataset),
                            prediction = round(score) == 1)
  
  fit = knn(dataset[,c("x","y")],dataset[,c("x","y")],dataset$train,k = 50,prob = T)
  models[["knn50"]] = mutate(dataset,
                             score = ifelse(fit=="FALSE",1-attr(fit,"prob"),attr(fit,"prob")),
                             prediction = round(score) == 1)
  
  tic("forest_ranger")
  fit = ranger(as.factor(train) ~ x + y, dataset,num.trees = 5000,probability=TRUE)
  toc()
  models[["forest_ranger"]] = mutate(dataset,
                              score = predict(fit,dataset)$predictions[,"TRUE"],
                              prediction = round(score) == 1)

  
  dataset.h2o = as.h2o(dataset,destination_frame = "dataset")
  
  tic("forest_h2o")
  hfit = h2o.randomForest(c("x","y"),
                          "train",
                          dataset.h2o,
                          model_id = "model",
                          ntrees = 5000)
  toc()
  models[["forest_h2o"]] = mutate(dataset,
                               score =  as.data.frame(predict(hfit,dataset.h2o))[,"TRUE."],
                               prediction = round(score) == 1)
  
  
  tic("nnet_50")
  hfit = h2o.deeplearning(c("x","y"),"class",
                          dataset.h2o,
                          model_id = "model",
                          hidden = 50,
                          epochs = 20000)
  toc()
  models[["nnet_50"]] = mutate(dataset,
                               score =  as.data.frame(predict(hfit,dataset.h2o))[,"TRUE."],
                               prediction = round(score) == 1)
  
  tic("deep_10x5")
  hfit = h2o.deeplearning(c("x","y"),"class",
                          dataset.h2o,
                          model_id = "model",
                          hidden = c(10,10,10,10,10),
                          epochs = 20000)
  toc()
  models[["deep_10x5"]] = mutate(dataset,
                                 score =  as.data.frame(predict(hfit,dataset.h2o))[,"TRUE."],
                                 prediction = round(score) == 1)

  rocs_train = map(models,function(x) getROC(x$score,x$train))
  rocPlot = plotROC(rocs_train) + ggtitle(paste("ROC",set,"train"))
  ggsave(paste0(set,"-rocPlot_train.png"),rocPlot)
    
  rocs_test = map(models,function(x) getROC(x$score,x$class))
  rocPlot = plotROC(rocs_test) + ggtitle(paste("ROC",set,"test"))
  ggsave(paste0(set,"-rocPlot_test.png"),rocPlot)
  
  plots = map(names(models), function(m) {
    gr = ggplot(data = models[[m]], aes(x=x, y=y, fill=prediction)) + geom_tile() + ggtitle(m)
    ggsave(paste0(set,"-predict_",m,".png"))
    return(gr)
  })
  toc()
}

